# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import exceptions
from openstack import resource

from otcextensions.common import exc
from otcextensions.common import utils

_logger = _log.setup_logging('openstack')


class Resource(resource.Resource):

    service_expectes_json_type = False

    allow_get = False
    allow_list = False
    allow_create = False
    allow_update = False
    allow_delete = False

    @classmethod
    def _prepare_override_args(cls,
                               endpoint_override=None,
                               request_headers=None,
                               additional_headers=None,
                               requests_auth=None):
        """Prepare additional (override) arguments for the REST call

        :param endpoint_override: optional endpoint_override argument
        :param request_headers: original calculated request headers
        :param additional_headers: additional headers to be set into request

        :returns arguments dict
        """
        req_args = {}

        if cls.service_expectes_json_type:
            # Some services are stupid and expect
            # 'Content-Type': 'application/json' even for empty requests
            req_args['headers'] = {'Content-Type': 'application/json'}

        if additional_headers and request_headers:
            req_args['headers'] = utils.merge_two_dicts(
                additional_headers,
                request_headers)
        else:
            if additional_headers:
                req_args['headers'] = additional_headers
            if request_headers:
                req_args['headers'] = request_headers

        if endpoint_override:
            req_args['endpoint_override'] = endpoint_override

        if requests_auth:
            req_args['requests_auth'] = requests_auth

        return req_args

    def _translate_response(
            self, response, has_body=None, error_message=None,
            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        'DELETE' operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        exc.raise_from_response(response, error_message=error_message)
        if has_body:
            if response.status_code == 204:
                # Some bad APIs (i.e. DCS.Backup.List) return emptiness
                _logger.warn('API returned no content, while it was expected')
                return
            body = response.json()
            if self.resource_key and self.resource_key in body:
                body = body[self.resource_key]

            body = self._consume_body_attrs(body)
            self._body.attributes.update(body)
            self._body.clean()

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()

    def create(self, session, prepend_key=True,
               endpoint_override=None, headers=None, requests_auth=None):
        """Create a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource creation
                            request. Default to True.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_create` is not set to ``True``.
        """
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        session = self._get_session(session)

        if self.create_method == 'PUT':
            request = self._prepare_request(requires_id=True,
                                            prepend_key=prepend_key)
            req_args = self._prepare_override_args(
                endpoint_override=endpoint_override,
                request_headers=request.headers,
                additional_headers=headers,
                requests_auth=requests_auth)
            response = session.put(request.url,
                                   json=request.body, **req_args)
        elif self.create_method == 'POST':
            request = self._prepare_request(requires_id=False,
                                            prepend_key=prepend_key)
            req_args = self._prepare_override_args(
                endpoint_override=endpoint_override,
                request_headers=request.headers,
                additional_headers=headers,
                requests_auth=requests_auth)
            response = session.post(request.url,
                                    json=request.body, **req_args)
        else:
            raise exceptions.ResourceFailure(
                msg="Invalid create method: %s" % self.create_method)

        self._translate_response(response)

        return self

    def get(self, session, error_message=None, requires_id=True,
            endpoint_override=None, headers=None, requests_auth=None):
        """Get a remote resource based on this instance.

        This function overrides default Resource.get to enable GET headers

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param boolean requires_id: A boolean indicating whether resource ID
                                    should be part of the requested URI.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_get` is not set to ``True``.
        """
        if not self.allow_get:
            raise exceptions.MethodNotSupported(self, "get")

        request = self._prepare_request(requires_id=requires_id)
        session = self._get_session(session)

        # Build additional arguments to the GET call
        get_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)

        response = session.get(request.url, **get_args)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, **kwargs)

        return self

    def head(self, session,
             endpoint_override=None, headers=None, requests_auth=None):
        """Get headers from a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_head` is not set to ``True``.
        """
        if not self.allow_head:
            raise exceptions.MethodNotSupported(self, "head")

        request = self._prepare_request()

        session = self._get_session(session)

        args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers={"Accept": ""},
            additional_headers=headers,
            requests_auth=requests_auth)

        response = session.head(request.url,
                                **args)

        self._translate_response(response, has_body=False)
        return self

    def update(self, session, prepend_key=True, has_body=True,
               endpoint_override=None, headers=None, requests_auth=None):
        """Update the remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource update request.
                            Default to True.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_update` is not set to ``True``.
        """
        # The id cannot be dirty for an update
        self._body._dirty.discard("id")

        # Only try to update if we actually have anything to update.
        if not any([self._body.dirty, self._header.dirty]):
            return self

        if not self.allow_update:
            raise exceptions.MethodNotSupported(self, "update")

        request = self._prepare_request(prepend_key=prepend_key)
        session = self._get_session(session)

        args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)

        if self.commit_method == 'PATCH':
            response = session.patch(
                request.url, json=request.body, **args)
        elif self.commit_method == 'POST':
            response = session.post(
                request.url, json=request.body, **args)
        elif self.commit_method == 'PUT':
            response = session.put(
                request.url, json=request.body, **args)
        else:
            raise exceptions.ResourceFailure(
                msg="Invalid update method: %s" % self.commit_method)

        self._translate_response(response, has_body=has_body)
        return self

    def delete(self, session, error_message=None,
               endpoint_override=None, headers=None,
               requests_auth=None, params=None):
        """Delete the remote resource based on this instance.

        This function overrides default Resource.delete to enable headers

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_update` is not set to ``True``.
        """
        if not self.allow_delete:
            raise exceptions.MethodNotSupported(self, "delete")

        request = self._prepare_request()
        session = self._get_session(session)

        # Build additional arguments to the DELETE call
        delete_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)
        if params:
            delete_args['params'] = params

        response = session.delete(request.url,
                                  **delete_args)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, has_body=False, **kwargs)
        return self

    @classmethod
    def list(cls, session, paginated=False,
             endpoint_override=None, headers=None, requests_auth=None,
             **params):
        """Override default list to incorporate endpoint overriding
        and custom headers

        Since SDK Resource.list method is passing hardcoded headers
        do override the function

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
                               a paginated series of responses, or ``False``
                               if a GET returns only one page of data.
                               **When paginated is False only one
                               page of data will be returned regardless
                               of the API's support of pagination.**
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be
            sent in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the
            :data:`~openstack.resource.Resource.base_path` format string
            to see if any path fragments need to be filled in by the contents
            of this argument.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
                 contains invalid params.

        """

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        session = cls._get_session(session)

        cls._query_mapping._validate(params, base_path=cls.base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = cls.base_path % params

        limit = query_params.get('limit')

        # Build additional arguments to the GET call
        get_args = cls._prepare_override_args(
            endpoint_override=endpoint_override,
            # request_headers=request.headers,
            additional_headers=headers)

        total_yielded = 0
        while uri:
            response = session.get(
                uri,
                params=query_params.copy(),
                **get_args
            )
            exceptions.raise_from_response(response)
            if response.status_code == 204 or \
                    (response.status_code == 200 and not response.json()):
                # Some bad APIs (i.e. DCS.Backup.List) return emptiness
                return
            data = response.json()

            # Discard any existing pagination keys
            query_params.pop('marker', None)
            query_params.pop('limit', None)

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                raw_resource.pop("self", None)

                if cls.resource_key and cls.resource_key in raw_resource:
                    raw_resource = raw_resource[cls.resource_key]

                value = cls.existing(**raw_resource)

                marker = value.id
                yield value
                total_yielded += 1

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, marker, limit, total_yielded)
                query_params.update(next_params)
            else:
                return

    @staticmethod
    def find_value_by_accessor(input_dict, accessor):
        """Gets value from a dictionary using a dotted accessor"""
        current_data = input_dict
        for chunk in accessor.split('.'):
            if isinstance(current_data, dict):
                current_data = current_data.get(chunk, {})
            else:
                return None
        return current_data

    @classmethod
    def list_ext(cls, session, paginated=False,
                 endpoint_override=None, headers=None, **params):
        """Override default list to incorporate endpoint overriding
        and custom headers

        Since SDK Resource.list method is passing hardcoded headers
        do override the function

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
                               a paginated series of responses, or ``False``
                               if a GET returns only one page of data.
                               **When paginated is False only one
                               page of data will be returned regardless
                               of the API's support of pagination.**
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be
            sent in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the
            :data:`~openstack.resource.Resource.base_path` format string
            to see if any path fragments need to be filled in by the contents
            of this argument.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
                 contains invalid params.

        """
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        session = cls._get_session(session)

        # pop scaling_group_id, as it should not be also present in the query
        scaling_group_id = params.pop('scaling_group_id', None)
        uri_params = {}

        if scaling_group_id:
            uri_params = {'scaling_group_id': scaling_group_id}

        print('got params %s' % params)

        cls._query_mapping._validate(params, base_path=cls.base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = None
        if not hasattr(cls, 'list_path'):
            uri = cls.base_path % uri_params
        else:
            uri = cls.list_path % uri_params

        limit = query_params.get('limit')

        # Build additional arguments to the GET call
        get_args = cls._prepare_override_args(
            endpoint_override=endpoint_override,
            # request_headers=request.headers,
            additional_headers=headers)

        total_yielded = 0
        while uri:
            response = session.get(
                uri,
                params=query_params.copy(),
                **get_args
            )
            exceptions.raise_from_response(response)
            data = response.json()

            # Discard any existing pagination keys
            query_params.pop('marker', None)
            query_params.pop('limit', None)

            if cls.resources_key:
                resources = cls.find_value_by_accessor(data, cls.resources_key)
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                raw_resource.pop("self", None)

                if cls.resource_key and cls.resource_key in raw_resource:
                    raw_resource = raw_resource[cls.resource_key]

                value = cls.existing(**raw_resource)

                marker = value.id
                yield value
                total_yielded += 1

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, marker, limit, total_yielded)
                query_params.update(next_params)
            else:
                return

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True,
             endpoint_override=None, headers=None, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.
        try:
            match = cls.existing(
                id=name_or_id,
                **params)
            return match.get(
                session,
                endpoint_override=endpoint_override,
                headers=headers)
        except (exceptions.NotFoundException, exceptions.HttpException,
                exceptions.MethodNotSupported):
            pass

        data = cls.list(session,
                        endpoint_override=endpoint_override,
                        headers=headers,
                        **params)

        result = cls._get_one_match(name_or_id, data)
        # Update result with URL parameters
        if result is not None:
            result._update(**params)
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))

    def update_no_id(self, session, prepend_key=True, has_body=True,
                     endpoint_override=None, headers=None):
        """Update the remote resource based on this instance.

        Method is required for resources without ID
        (single resource at endpoint)

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource update request.
                            Default to True.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_update` is not set to ``True``.
        """
        # Only try to update if we actually have anything to update.
        if not any([self._body.dirty, self._header.dirty]):
            return self

        if not self.allow_update:
            raise exceptions.MethodNotSupported(self, "update")

        request = self._prepare_request(
            requires_id=False,
            prepend_key=prepend_key)
        session = self._get_session(session)

        update_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers)

        if self.commit_method == 'PATCH':
            response = session.patch(
                request.url, json=request.body, **update_args)
        elif self.commit_method == 'POST':
            response = session.post(
                request.url, json=request.body, **update_args)
        elif self.commit_method == 'PUT':
            response = session.put(
                request.url, json=request.body, **update_args)
        else:
            raise exceptions.ResourceFailure(
                msg="Invalid update method: %s" % self.commit_method)

        self._translate_response(response, has_body=has_body)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Eip(resource.Resource):
    base_path = '/nodes'

    resources_key = 'nodes'
    resource_key = 'node'

    job_id = resource.Body('job_id')
    node_id = resource.Body('node_id')
    node_name = resource.Body('node_name')
    public_ip = resource.Body('public_ip')
    public_ip_id = resource.Body('public_ip_id')

    def bind(self, session, node, public_ip, public_ip_id):
        body = {
            "public_ip": public_ip,
            "public_ip_id": public_ip_id
        }
        response = self._action(session, body, f'{node}/bind-eip')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def unbind(self, session, node):
        response = self._action(session, {}, f'{node}/unbind-eip')
        self._translate_response(response)
        return self

    def _action(self, session, body, action_type):
        """Preform actions given the message body.
        """
        url = utils.urljoin(self.base_path, action_type)
        return session.post(
            url,
            json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG
import uuid


class TestGateway(TestApiG):
    eip = None
    admin_external_net = None

    def setUp(self):
        super(TestGateway, self).setUp()
        self.create_gateway()

    def tearDown(self):
        super(TestGateway, self).tearDown()

    def test_01_list_gateways(self):
        attrs = {
            'limit': 2
        }
        gateways = list(self.client.gateways(**attrs))
        self.assertGreater(len(gateways), 0)

    def test_02_creation_progress(self):
        found = self.client.get_gateway_progress(TestGateway.gateway)
        self.assertIsNotNone(found.progress)

    def test_03_get_gateway(self):
        found = self.client.get_gateway(TestGateway.gateway.instance_id)
        self.assertEqual(found.id, TestGateway.gateway.instance_id)
        TestGateway.gateway = found

    def test_04_update_gateway(self):
        self.client.wait_for_gateway(TestGateway.gateway)
        test_nmb = uuid.uuid4().hex[:8]
        attrs = {
            "description": test_nmb
        }
        found = self.client.update_gateway(TestGateway.gateway, **attrs)
        self.assertEqual(found.description, attrs['description'])

    def test_05_get_constraints(self):
        found = self.client.get_constraints(TestGateway.gateway.id)
        self.assertIsNotNone(found.resource_subnet_cidr)

    def test_06_enable_public_access(self):
        self.client.wait_for_gateway(TestGateway.gateway)
        attrs = {
            "bandwidth_size": "5",
            "bandwidth_charging_mode": "bandwidth"
        }
        found = self.client.enable_public_access(TestGateway.gateway.id,
                                                 **attrs)
        self.assertIsNotNone(found.bandwidth_name)

    def test_07_update_public_access(self):
        self.client.wait_for_gateway(TestGateway.gateway)
        attrs = {
            "bandwidth_size": "7",
            "bandwidth_charging_mode": "bandwidth"
        }
        found = self.client.update_public_access(TestGateway.gateway.id,
                                                 **attrs)
        self.assertIsNotNone(found.bandwidth_name)

    def test_08_disable_public_access(self):
        self.client.wait_for_gateway(TestGateway.gateway)
        self.client.disable_public_access(TestGateway.gateway.id)

    # def test_09_modify_spec(self):
    #     self.client.wait_for_gateway(TestGateway.gateway)
    #     attrs = {
    #         "spec_id" : "PROFESSIONAL"
    #     }
    #     found = self.client.modify_gateway_spec(TestGateway.gateway, **attrs)
    #     self.assertIsNotNone(found.job_id)

    # def test_10_enable_ingress(self):
    #     self.client.wait_for_gateway(TestGateway.gateway)
    #     attrs = {
    #         "bandwidth_size" : "5",
    #         "bandwidth_charging_mode" : "bandwidth"
    #     }
    #     found = self.client.enable_ingress(TestGateway.gateway.id, **attrs)
    #     self.assertIsNotNone(found.job_id)
    #
    # def test_11_update_ingress(self):
    #     self.client.wait_for_gateway(TestGateway.gateway)
    #     attrs = {
    #         "bandwidth_size" : "7",
    #         "bandwidth_charging_mode" : "bandwidth"
    #     }
    #     found = self.client.update_ingress(TestGateway.gateway.id, **attrs)
    #     self.assertIsNotNone(found.job_id)
    #
    # def test_12_disable_ingress(self):
    #     self.client.wait_for_gateway(TestGateway.gateway)
    #     self.client.disable_ingress(TestGateway.gateway.id)

    # def test_13_bind_eip(self):
    #     admin_external_net = self.conn.network.find_network(
    #         name_or_id='admin_external_net')
    #     self.assertIsNotNone(admin_external_net)
    #     floating_ip = self.conn.network.create_ip(
    #         floating_network_id=admin_external_net.id)
    #     self.client.wait_for_gateway(TestGateway.gateway)
    #     attrs = {
    #         "eip_id": floating_ip.id
    #     }
    #     TestGateway.eip = floating_ip
    #     TestGateway.admin_external_net = admin_external_net
    #     self.client.bind_eip(TestGateway.gateway, **attrs)
    #
    # def test_14_unbind_eip(self):
    #     self.client.unbind_eip(TestGateway.gateway)
    #     self.conn.network.delete_ip(TestGateway.eip)

    def test_15_delete_gateway(self):
        self.client.delete_gateway(TestGateway.gateway)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from cliff import columns as cliff_columns

from openstackclient.tests.unit import utils


class TestCommand(utils.TestCommand):
    """Test cliff command classes"""

    def assertListItemEqual(self, expected, actual):
        self.assertEqual(len(expected), len(actual))
        for item_expected, item_actual in zip(expected, actual):
            self.assertItemEqual(item_expected, item_actual)

    def assertItemEqual(self, expected, actual):
        self.assertEqual(len(expected), len(actual))
        for col_expected, col_actual in zip(expected, actual):
            if isinstance(col_expected, cliff_columns.FormattableColumn):
                self.assertIsInstance(col_actual, col_expected.__class__)
                self.assertEqual(col_expected.human_readable(),
                                 col_actual.human_readable())
            else:
                self.assertEqual(col_expected, col_actual)


class Fake(object):

    @classmethod
    def create_one(cls, attrs=None):
        """Create a fake resource.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        attrs = attrs or {}

        resource = cls.generate()

        return resource

    @classmethod
    def create_multiple(cls, count=2, attrs=None):
        """Create multiple fake resources.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of address scopes to fake
        :return:
            A list of FakeResource objects faking the address scopes
        """
        objects = []
        for i in range(0, count):
            objects.append(
                cls.create_one(attrs))

        return objects



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.cbr.v3 import policy as _policy


EXAMPLE = {
    'enabled': True,
    'name': 'my_policy',
    'operation_definition': {
        'day_backups': 0,
        'month_backups': 0,
        'max_backups': 1,
        'timezone': 'UTC+08:00',
        'week_backups': 0,
        'year_backups': 0
    },
    'operation_type': 'backup',
    'trigger': {
        'properties': {
            'pattern': [
                'FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR,SA,SU;BYHOUR=14;BYMINUTE=00'
            ]
        }
    }
}


class TestPolicy(base.TestCase):

    def setUp(self):
        super(TestPolicy, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.list = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _policy.Policy()
        self.sot_expected = _policy.Policy(**EXAMPLE)

    def test_basic(self):
        sot = _policy.Policy()
        self.assertEqual('policy', sot.resource_key)
        self.assertEqual('policies', sot.resources_key)
        self.assertEqual('/policies',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        test_policy = _policy.Policy(**EXAMPLE)
        self.assertTrue(test_policy.enabled)
        self.assertEqual(
            EXAMPLE['name'],
            test_policy.name)
        self.assertEqual(
            EXAMPLE['operation_type'],
            test_policy.operation_type)
        self.assertEqual(
            EXAMPLE['operation_definition']['day_backups'],
            test_policy.operation_definition.day_backups)
        self.assertEqual(
            EXAMPLE['operation_definition']['month_backups'],
            test_policy.operation_definition.month_backups)
        self.assertEqual(
            EXAMPLE['operation_definition']['max_backups'],
            test_policy.operation_definition.max_backups)
        self.assertEqual(
            EXAMPLE['operation_definition']['timezone'],
            test_policy.operation_definition.timezone)
        self.assertEqual(
            EXAMPLE['operation_definition']['week_backups'],
            test_policy.operation_definition.week_backups)
        self.assertEqual(
            EXAMPLE['operation_definition']['year_backups'],
            test_policy.operation_definition.year_backups)
        self.assertEqual(
            EXAMPLE['trigger']['properties']['pattern'][0],
            test_policy.trigger.properties.pattern[0])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.smn.v2 import topic as _topic
from otcextensions.sdk.smn.v2 import subscription as _subscription
from otcextensions.sdk.smn.v2 import template as _template
from otcextensions.sdk.smn.v2 import message as _message
from otcextensions.sdk.smn.v2 import sms as _sms

from openstack import proxy


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Topic ========
    def create_topic(self, **attrs):
        """Create a new topic from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        :returns: :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        """
        return self._create(_topic.Topic, **attrs)

    def delete_topic(self, topic, ignore_missing=True):
        """Delete a topic

        :param topic: topic urn or an instance of
            :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the topic does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent topic.
        :returns: ``None``
        """
        return self._delete(_topic.Topic, topic,
                            ignore_missing=ignore_missing)

    def topics(self, **query):
        """Return a generator of SMN topics

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of topic objects
        :rtype: :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        """
        return self._list(_topic.Topic, **query)

    def get_topic(self, topic):
        """Get details a SMN topic

        :param topic: The value can be the ID of a topic or a
                        :class:`~otcextensions.sdk.smn.v2.topic.Topic`
                        instance.
        :returns: One :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_topic.Topic, topic)

    def find_topic(self, name_or_id, ignore_missing=False):
        """Find a SMN topic

        :param name_or_id: The name or ID of a topic.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the template does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent template.

        :returns: One :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        """
        return self._find(_topic.Topic, name_or_id,
                          ignore_missing=ignore_missing)

    def update_topic(self, topic, **attrs):
        """Update a SMN topic

        :param topic: Either the ID of a topic or a
                       :class:`~otcextensions.sdk.smn.v2.topic.Topic`
                       instance.
        :param dict attrs: The attributes to update on the topic represented
                       by :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        :returns: The updated topic.
        :rtype: :class:`~otcextensions.sdk.smn.v2.topic.Topic`
        """
        return self._update(_topic.Topic, topic, **attrs)

    # ======== Topic Attributes (Access Policy)========
    """
        The Topic Attributes are not well designed via API. So the SDK
        implementation sucks as well. We need for a proper API fix to get
        this solved
    """

    def topic_attributes(self, topic, **query):
        """Get SMN topic attributes

        :param topic: The value can be the ID of a topic or a
            :class:`~otcextensions.sdk.smn.v2.topic.Topic`
            instance.
        :param query: Attribute query params.
        :returns: One :class:`~otcextensions.sdk.smn.v2.topic.TopicAttribute`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        topic = self._get_resource(_topic.Topic, topic)
        return self._list(
            _topic.TopicAttribute,
            topic_id=topic.id,
            **query)

    def update_topic_attribute(self, topic, name='access_policy', **attrs):
        """Update SMN topic attributes

        :param topic: Either the ID of a topic or a
            :class:`~otcextensions.sdk.smn.v2.topic.Topic`
            instance.
        :param name: Attribute Name.
        :param dict attrs: The attributes to update on the topic represented
            by :class:`~otcextensions.sdk.smn.v2.topic.TopicAttribute`
        :returns: request_id.

        :rtype: :class:`~otcextensions.sdk.smn.v2.topic.TopicAttribute`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        topic = self._get_resource(_topic.Topic, topic)
        return self._update(_topic.TopicAttribute, id=name,
                            topic_id=topic.id, **attrs)

    def delete_topic_attribute(self, topic, name=None):
        """Delete all attributes of a topic

        :param topic: Either the ID of a topic or a
            :class:`~otcextensions.sdk.smn.v2.topic.Topic`
            instance.
        :param name: Attribute Name.
        :returns: request_id.
        :rtype: :class:`~otcextensions.sdk.smn.v2.topic.TopicAttribute`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        topic = self._get_resource(_topic.Topic, topic)
        if name:
            return self._delete(_topic.TopicAttribute,
                                id=name, topic_id=topic.id)
        return self._delete(_topic.TopicAttribute,
                            topic_id=topic.id, requires_id=False)

    # ======== Subscription ========
    def create_subscription(self, topic, **attrs):
        """Create a new Subscription from attributes

        :param topic: Either the ID of a topic or a
            :class:`~otcextensions.sdk.smn.v2.topic.Topic`
            instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.smn.v2.subscription.Subscription`
        :returns: :class:`~otcextensions.sdk.smn.v2.subscription.Subscription`
        """
        topic = self._get_resource(_topic.Topic, topic)
        return self._create(_subscription.Subscription,
                            topic_urn=topic.id, **attrs)

    def delete_subscription(self, subscription, ignore_missing=True):
        """Delete a subscription

        :param subscription: Either the ID of a subscription or a
            :class: `~otcextensions.sdk.smn.v2.subscription.Subscription`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the subscription does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent subscription.
        :returns: ``None``
        """
        return self._delete(_subscription.Subscription, subscription,
                            ignore_missing=ignore_missing)

    def subscriptions(self, topic=None, **query):
        """Return a generator of Subscriptions

        :param topic: Either the ID of a topic or a
            :class: `~otcextensions.sdk.smn.v2.topic.Topic`
            instance.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of subscription objects
        :rtype: :class:`~otcextensions.sdk.smn.v2.subscription.Subscription`
        """
        if topic:
            topic = self._get_resource(_topic.Topic, topic)
            return self._list(_subscription.Subscription,
                              topic_urn=topic.id, **query)
        return self._list(_subscription.Subscription, **query)

    # ======== Template ========
    def create_template(self, **attrs):
        """Create a new message template from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.smn.v2.template.Template`

        :returns: :class:`~otcextensions.sdk.smn.v2.template.Template`
        """
        return self._create(_template.Template, **attrs)

    def delete_template(self, template, ignore_missing=True):
        """Delete a message template

        :param template: message template ID or an instance of
            :class:`~otcextensions.sdk.smn.v2.template.Template`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the template does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent template.

        :returns: ``None``
        """
        return self._delete(_template.Template, template,
                            ignore_missing=ignore_missing)

    def templates(self, **query):
        """Return a generator of message templates

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of template objects.

        :rtype: :class:`~otcextensions.sdk.smn.v2.template.Template`
        """
        return self._list(_template.Template, **query)

    def get_template(self, template):
        """Get details a message templates

        :param template: The value can be the ID of a message template or a
            :class:`~otcextensions.sdk.smn.v2.template.Template` instance.

        :returns: One :class:`~otcextensions.sdk.smn.v2.template.Template`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_template.Template, template)

    def update_template(self, template, **attrs):
        """Update a message template

        :param topic: Either the ID of a message template or a
            :class:`~otcextensions.sdk.smn.v2.template.Template` instance.
        :param dict attrs: The attributes to update on the template represented
            by :class:`~otcextensions.sdk.smn.v2.template.Template`

        :returns: The updated template.

        :rtype: :class:`~otcextensions.sdk.smn.v2.template.Template`
        """
        return self._update(_template.Template, template, **attrs)

    def find_template(self, name_or_id, ignore_missing=False):
        """Find a single message template.

        :param name_or_id: The name or ID of a template.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the template does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent template.

        :returns: One :class:`~otcextensions.sdk.smn.v2.template.Template`
        """
        return self._find(_template.Template, name_or_id,
                          ignore_missing=ignore_missing)

    # ======== Message Publish ========
    def publish_message(self, topic, **attrs):

        """
        Publish messages in the text format or
        using message structure or using a message template
        to a topic.

        :param topic: Either the ID of a topic or a
            :class: `~otcextensions.sdk.smn.v2.topic.Topic`
            instance.
        :param dict attrs: Keyword arguments which will be used to Publish
            a Message.

        :returns: :class:`~otcextensions.sdk.smn.v2.message.Message`
        """
        topic = self._get_resource(_topic.Topic, topic)
        return self._create(_message.Message,
                            topic_urn=topic.id, **attrs)

    # ======== SMS Publish ========
    def send_sms(self, endpoint, message):

        """
        Send a transactional SMS message to a specified phone number,
        usually used for verification code or notification.

        :param endpoint: Phone number.
        :param message: SMS message content.

        :returns: :class:`~otcextensions.sdk.smn.v2.message.Message`
        """
        attrs = {
            'endpoint': endpoint,
            'message': message
        }
        return self._create(_sms.Sms, **attrs)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class CceMixin:

    # ===== CCE Cluster Operations =====

    def create_cce_cluster(
        self,
        wait=True, wait_timeout=1800, wait_interval=5,
        **kwargs
    ):
        """Create CCE cluster

        :param str name: Cluster name.
        :param str type: Cluster type.
        :param str flavor: Flavor.
        :param str router: Name or ID of the neutron router.
        :param str network: Name or ID of the netron network.
        :param dict annotations: Annotations.
        :param str authentication_proxy_ca: CA for the authentication proxy.
        :param str authentication_mode: Authentication mode.
        :param str availability_zone: Whether to support multi_az master
            distribution.
        :param str billing_mode: Not supported.
        :param str container_network_mode: Network mode.
        :param str container_network_cidr: CIDR for the internal network.
        :param str cpu_manager_policy: CPU policy.
        :param str dss_master_volumes: DSS master volumes.
        :param str description: Cluster description.
        :param str external_ip: IP address.
        :param str fix_pool_mask: Not supported.
        :param dict labels: Option labels.
        :param str service_ip_rance: Service IP Range.
        :param str kube_proxy_mode: Proxy mode.
        :param str upgrade_from: Not used.
        :param str version: Kubernetes version
        :param bool wait: dict(type=bool, default=True),
        :param int wait_timeout: dict(type=int, default=180)
        :param int wait_interval: Check interval.

        :returns: The results of cluster creation
        :rtype: :class:`~otcextensions.sdk.cce.v3.cluster.Cluster`
        """
        name = kwargs.get('name')
        cluster_type = kwargs.get('type')
        flavor = kwargs.get('flavor')
        router = kwargs.get('router')
        network = kwargs.get('network')
        annotations = kwargs.get('annotations')
        authentication_proxy_ca = kwargs.get('authentication_proxy_ca')
        authentication_mode = kwargs.get('authentication_mode')
        az = kwargs.get('availability_zone')
        billing_mode = kwargs.get('billing_mode')
        container_network_cidr = kwargs.get('container_network_cidr')
        container_network_mode = kwargs.get('container_network_mode')
        cpu_manager_policy = kwargs.get('cpu_manager_policy')
        dss_master_volumes = kwargs.get('dss_master_volumes')
        description = kwargs.get('description')
        external_ip = kwargs.get('external_ip')
        fix_pool_mask = kwargs.get('fix_pool_mask')
        labels = kwargs.get('labels')
        service_ip_range = kwargs.get('service_ip_range')
        kube_proxy_mode = kwargs.get('kube_proxy_mode')
        upgrade_from = kwargs.get('upgrade_from')
        version = kwargs.get('version')

        metadata = {
            'name': name,
        }
        if labels and isinstance(labels, dict):
            metadata['labels'] = labels
        if annotations and isinstance(annotations, dict):
            metadata['annotations'] = annotations

        if cluster_type.lower() not in ('virtualmachine'):
            raise ValueError('CCE Cluster type %s is not supported' %
                             cluster_type)
        # Note(gtema): we do not validate flavor, since it is a subject of
        # constant change

        spec = {
            'type': cluster_type,
            'flavor': flavor,
        }

        if version:
            spec['version'] = version
        if description:
            spec['description'] = description
        if service_ip_range:
            spec['kubernetesSvcIpRange'] = service_ip_range
        if kube_proxy_mode:
            spec['kubeProxyMode'] = kube_proxy_mode
        if billing_mode:
            spec['billingMode'] = billing_mode
        if authentication_mode or authentication_proxy_ca:
            auth = {}
            if (
                authentication_mode
                and authentication_mode in (
                    'x509', 'rbac', 'authenticating_proxy')
            ):
                auth['mode'] = authentication_mode
            elif authentication_mode:
                raise ValueError(
                    'Authentication mode %s is not supported'
                    % authentication_mode)
            if authentication_proxy_ca:
                auth['authenticatingProxy'] = {'ca': authentication_proxy_ca}
            spec['authentication'] = auth

        container_network = {}

        if container_network_mode not in ('overlay_l2', 'underlay_ipvlan',
                                          'vpc-router'):
            raise ValueError(
                'Container network mode %s is not supported' %
                container_network_mode
            )
        container_network['mode'] = container_network_mode
        if container_network_cidr:
            container_network['cidr'] = container_network_cidr
        spec['containerNetwork'] = container_network

        extended_params = {}
        if az:
            extended_params['clusterAZ'] = az
        if dss_master_volumes:
            extended_params['dssMasterVolumes'] = dss_master_volumes
        if external_ip:
            extended_params['clusterExternalIP'] = external_ip
        if fix_pool_mask:
            extended_params['alpha.cce/fixPoolMask'] = fix_pool_mask
        if cpu_manager_policy:
            extended_params['kubernetes.io/cpuManagerPolicy'] = \
                cpu_manager_policy
        if upgrade_from:
            extended_params['upgradefrom'] = upgrade_from

        if extended_params:
            spec['extendedParam'] = extended_params

        router_obj = self.network.find_router(
            router, ignore_missing=False)
        network_obj = self.network.find_network(
            network, ignore_missing=False)
        host_network = {
            'vpc': router_obj.id,
            'subnet': network_obj.id
        }
        spec['hostNetwork'] = host_network

        obj = self.cce.create_cluster(
            metadata=metadata,
            spec=spec
        )

        if obj.job_id and wait:
            wait_args = {}
            if wait_interval:
                wait_args['interval'] = wait_interval
            if wait_timeout:
                wait_args['wait'] = wait_timeout

            self.cce.wait_for_job(obj.job_id, **wait_args)
            obj = self.cce.get_cluster(obj.id)

        return obj

    def delete_cce_cluster(
        self,
        wait=True, wait_timeout=1800, wait_interval=5,
        **kwargs
    ):
        """Delete CCE cluster

        :param str cluster: Name or ID of the cluster
        """
        cluster = kwargs.get('cluster')
        obj = self.cce.find_cluster(name_or_id=cluster, ignore_missing=False)
        obj = self.cce.delete_cluster(obj.id)

        if obj.job_id and wait:
            wait_args = {}
            if wait_interval:
                wait_args['interval'] = wait_interval
            if wait_timeout:
                wait_args['wait'] = wait_timeout

            self.cce.wait_for_job(obj.job_id, **wait_args)

        return None

    # ===== CCE Cluster Node Operations =====

    def create_cce_cluster_node(
        self,
        count=1,
        root_volume_size=40,
        root_volume_type='SATA',
        data_volumes=[{
            'volumetype': 'SATA',
            'size': 100,
            'encrypted': False,
            'cmk_id': ''
        }],
        wait=True, wait_timeout=300, wait_interval=5,
        **kwargs
    ):
        """Create CCE cluster node

        :param dict annotations: Annotations.
        :param str availability_zone: Availability zone of the cluster_node.
        :param int bandwidth: Bandwidth size for the floating IPs being
            created. Must be used together with fip_count.
        :param int count: Count of the cluster nodes to be created.
        :param str cluster: CCE cluster attached to.
        :param list data_volumes: Data disk parameters of a node. At
            present, only one data disk can be configured. The list must have
            the following structure while parameter encrypted and cmk_id are
            optional:
            [{
                'volumetype': 'SATA',
                'size': 100,
                'encrypted': False,
                'cmk_id': ''
            },]
        :param str dedicated_host: Name or ID of the Dedicated Host to which
            nodes will be scheduled.
        :param str ecs_group: ID of the ECS group where the CCE node can
            belong to.
        :param str fault_domain: The node is created in the specified fault
            domain.
        :param int fip_count: Count of floating ip addresses to be attached.
        :param str flavor: Flavor ID of the CCE node.
        :param list floating_ips: List of Floating IP addresses or IDs to
            be attached to the nodes. The count of the CCE nodes and the
            Floating IP count must be equal.
        :param dict k8s_tags: Dictionary of Kubernetes tags.
        :param dict labels: Option labels.
        :param str lvm_config: ConfigMap of the Docker data disk.
        :param int max_pods: Maximum number of pods on the node.
        :param str name: Cluster node name.
        :param str network: ID of the network of the node.
        :param str node_image_id: ID of a custom image used in a bare metal
            scenario.
        :param str os: Operating system of the cluster node.
        :param str postinstall_script: Base64 encoded post installation
            script.
        :param str preinstall_script: Base64 encoded pre installation script.
        :param int root_volume_size: Size of the root volume.
        :param str root_volume_type: Type of the root volume.
        :param str ssh_key: Keypair to login into the node.
        :param list tags: List of tags used to build UI labels in format
            [{
                'key': 'key1',
                'value': 'value1
            },{
                'key': 'key2',
                'value': 'value2
            }]
        :param bool wait: Wait for node creation.
        :param int wait_timeout: Timeout for node creation. Wait must be True.
        :param int wait_interval: Check interval for node creation status.
            Wait must be True.

        :returns: The results of cluster node creation
        :rtype: :class:`~otcextensions.sdk.cce.v3.cluster_node.ClusterNode`
        """
        annotations = kwargs.get('annotations')
        availability_zone = kwargs.get('availability_zone')
        bandwidth = kwargs.get('bandwidth')
        cce_cluster = kwargs.get('cluster')
        dedicated_host = kwargs.get('dedicated_host')
        ecs_group = kwargs.get('ecs_group')
        fault_domain = kwargs.get('fault_domain')
        fip_count = kwargs.get('fip_count')
        flavor = kwargs.get('flavor')
        floating_ips = kwargs.get('floating_ips')
        k8s_tags = kwargs.get('k8s_tags')
        labels = kwargs.get('labels')
        lvm_config = kwargs.get('lvm_override_config')
        max_pods = kwargs.get('max_pods')
        name = kwargs.get('name')
        network = kwargs.get('network')
        node_image_id = kwargs.get('node_image_id')
        os = kwargs.get('os')
        postinstall_script = kwargs.get('postinstall_script')
        preinstall_script = kwargs.get('preinstall_script')
        ssh_key = kwargs.get('ssh_key')
        tags = kwargs.get('tags')

        volume_types = ['SAS', 'SATA', 'SSD']

        metadata = {
            'name': name,
        }

        if annotations and isinstance(annotations, dict):
            metadata['annotations'] = annotations
        if labels and isinstance(labels, dict):
            metadata['labels'] = labels

        spec = {
            'dataVolumes': {},
            'extendParam': {},
            'k8sTags': {},
            'login': {},
            'nodeNicSpec': {
                'primaryNic': {}
            },
            'publicIP': {},
            'rootVolume': {},
            'userTags': []
        }

        if count and isinstance(count, int):
            if count < 0:
                raise ValueError('count is 0 or lower')
            spec['count'] = count
        spec['flavor'] = flavor
        spec['login']['sshKey'] = ssh_key

        # Root volume specs
        if root_volume_size:
            if not isinstance(root_volume_size, int):
                raise ValueError('root_volume_size is not an integer value')
            if root_volume_size < 40:
                raise ValueError('Root volume size %s is lower than 40 GB.'
                                 % root_volume_size)
            spec['rootVolume']['size'] = root_volume_size
        if root_volume_type.upper() not in volume_types:
            raise ValueError('Root volume type %s is not supported, use: %s'
                             % root_volume_type, volume_types)
        spec['rootVolume']['volumetype'] = root_volume_type.upper()

        # Data volume specs
        volumes = []
        for item in data_volumes:
            if not (item['volumetype'] and item['size']):
                raise ValueError('One or both data volume keys: volumetype '
                                 'or size are missing.')
            if item['volumetype'] not in volume_types:
                raise ValueError('The data volumes volumetype %s must be '
                                 'one of the following choices: %s'
                                 % item['volumetype'], volume_types)
            if not isinstance(item['size'], int):
                try:
                    item['size'] = int(item['size'])
                except ValueError:
                    print('data_volume size %s cannot be converted into '
                          'integer value.' % item['size'])
            if not (100 <= item['size'] <= 32768):
                raise ValueError('The data volume size must be specified '
                                 'between 100 and 32768 GB.')
            if ('encrypted' in item) and item['encrypted']:
                if not item['cmk_id']:
                    raise ValueError('Parameter cmk_id is missing to use '
                                     'data volume encryption.')
                volumes.append({
                    'volumetype': item['volumetype'].upper(),
                    'size': item['size'],
                    'metadata': {
                        '__system__encrypted': '1',
                        '__system__cmkid': item['cmk_id']
                    }
                })
            else:
                volumes.append({
                    'volumetype': item['volumetype'].upper(),
                    'size': item['size']
                })
        if volumes:
            spec['dataVolumes'] = volumes

        if availability_zone:
            spec['az'] = availability_zone
        if dedicated_host:
            deh = self.deh.find_host(
                name_or_id=dedicated_host,
                ignore_missing=False)
            spec['dedicatedHostId'] = deh.id
        if ecs_group:
            spec['ecsGroupId'] = ecs_group
        if fault_domain:
            spec['faultDomain'] = fault_domain
        if floating_ips and isinstance(floating_ips, list):
            ids = []
            if not (count == len(floating_ips)):
                raise ValueError('The length of the list floating_ip_ids is'
                                 'different from CCE node count.')
            for ip in floating_ips:
                ip = self.network.find_ip(name_or_id=ip, ignore_missing=False)
                ids.append(ip.id)
            if ids:
                spec['publicIP']['ids'] = ids
        if fip_count and bandwidth:
            if not isinstance(bandwidth, int):
                raise ValueError('bandwidth must be an integer value.')
            if not isinstance(fip_count, int):
                raise ValueError('fip_count must be an integer value.')
            if floating_ips:
                raise ValueError('floating_ips is used together with'
                                 'fip_count. These values are exclusive.')
            spec['publicIP'] = {
                'count': fip_count,
                'eip': {
                    'iptype': '5_bgp',
                    'bandwidth': {
                        'chargemode': 'traffic',
                        'size': bandwidth,
                        'sharetype': 'PER'
                    }
                }
            }
        if k8s_tags and isinstance(k8s_tags, dict):
            spec['k8sTags'] = k8s_tags
        if lvm_config:
            spec['extendParam']['DockerLVMConfigOverride'] = lvm_config
        if max_pods and isinstance(max_pods, int):
            spec['extendParam']['maxPods'] = max_pods
        if node_image_id:
            spec['extendParam']['alpha.cce/NodeImageID'] = node_image_id
        if os:
            spec['os'] = os
        if postinstall_script:
            spec['extendParam']['alpha.cce/preInstall'] = postinstall_script
        if preinstall_script:
            spec['extendParam']['alpha.cce/preInstall'] = preinstall_script
        if not network:
            raise ValueError('network missing.')
        nw = self.network.find_network(
            network, ignore_missing=False)
        spec['nodeNicSpec']['primaryNic']['subnetId'] = nw.id
        if tags:
            spec['userTags'] = tags

        cluster = self.cce.find_cluster(
            name_or_id=cce_cluster,
            ignore_missing=True)
        if not cluster:
            raise ReferenceError('Cluster %s not found.' % cce_cluster)
        obj = self.cce.create_cluster_node(
            cluster=cluster.id,
            metadata=metadata,
            spec=spec
        )

        if obj.job_id and wait:
            wait_args = {}
            if wait_interval:
                wait_args['interval'] = wait_interval
            if wait_timeout:
                wait_args['wait'] = wait_timeout

            self.cce.wait_for_job(obj.job_id, **wait_args)
            obj = self.cce.get_cluster_node(
                cluster=cluster.id,
                node_id=obj.id
            )

        return obj

    def delete_cce_cluster_node(
        self,
        wait=True, wait_timeout=180, wait_interval=5,
        **kwargs
    ):
        """Delete CCE cluster node

        :param str cluster: Name or ID of the CCE cluster
        :param str node: Name or ID of the CCE cluster node
        """
        cluster = kwargs.get('cluster')
        node = kwargs.get('node')

        cluster = self.cce.find_cluster(
            name_or_id=cluster,
            ignore_missing=False)
        node = self.cce.find_cluster_node(
            cluster=cluster,
            node=node)

        obj = self.cce.delete_cluster_node(
            cluster=cluster,
            node=node
        )

        if obj.job_id and wait:
            wait_args = {}
            if wait_interval:
                wait_args['interval'] = wait_interval
            if wait_timeout:
                wait_args['wait'] = wait_timeout

            self.cce.wait_for_job(obj.job_id, **wait_args)

        return None

    # ===== CCE Node Pool =====

    def create_cce_node_pool(
        self,
        autoscaling_enabled=False,
        availability_zone='random',
        billing_mode=0,
        data_volumes=[{
            'volumetype': 'SATA',
            'size': 100,
            'encrypted': False,
            'cmk_id': ''
        }],
        initial_node_count=0,
        node_pool_type='vm',
        root_volume_size=40,
        root_volume_type='SATA',
        **kwargs
    ):
        """Create CCE node pool

        :param str availability_zone: Name of the AZ where the node resides.
            If availability_zone is set to: random (default), nodes can be
            created in any AZ.
        :param str autoscaling_enabled: Enable or disable auto-scaling
        :param str billing_mode: Only pay per use is supported. Default: 0
        :param int count: Count of the cluster nodes to be created in Batch.
            Can be 0 for a CCE node pool.
        :param str cluster: Name or ID of the CCE to which the node pool
            belongs to.
        :param list data_volumes: Data disk parameters of a node. At
            present, only one data disk can be configured. The list must have
            the following structure while parameter encrypted and cmk_id are
            optional:
            [{
                'volumetype': 'SATA',
                'size': 100,
                'encrypted': False,
                'cmk_id': ''
            },]
        :param str ecs_group: ECS group id of the ECS group to which nodes
            belong after creation
        :param str flavor: Flavor ID of the CCE node. Mandatory Parameter.
        :param int initial_node_count: Expected number of nodes in this
            node pool
        :param dict k8s_tags: Dictionary of Kubernetes tags.
        :param str lvm_config: Config Map of the Docker data disk
        :param int min_node_count: Minimum number of nodes after scale-up
        :param int max_node_count: Maximum number of nodes after scale-up
        :param int max_pods: Maximum number of pods on the node
        :param str name: Name of the node pool.
        :param str node_image_id: Mandatory if custom image is used on a
            bare metall node
        :param str node_pool_type: Type of the node pool. Currently, only
            'vm' is supported.
        :param str os: Operating system of the cluster node.
        :param str preinstall_script: Script required before installation.
            Input must be Base64 encoded.
        :param str postinstall_script: Script required after installation.
            Input must be Base64 encoded.
        :param int priority: Node pool weight for scale-up operations.
        :param str public_key: Additional public key to be added for login.
        :param int root_volume_size: Root volume size in GB
        :param str root_volume_type: Volume type; available option: SATA,
            SAS, SSD.
        :param int scale_down_cooldown_time: Interval in minutes during
            which nodes added after a scale-up will not be deleted.
        :param str ssh_key: SSH public key name for login in the created
            nodes
        :param str network: ID of the network to which the CCE node pool
            belongs
        :param list tags: List of tags used to build UI labels in format
            [{
                'key': 'key1',
                'value': 'value1
            },{
                'key': 'key2',
                'value': 'value2
            }]
        :param list taints: List of taints
            [{
                'key': 'key1',
                'value': 'value1,
                'effect': 'NoSchedule
            },{
                'key': 'key2',
                'value': 'value2,
                'effect': 'NoSchedule
            }]
            Effect available options: NoSchedule,
            PreferNoSchedule, NoExecute.

        :returns: The results of CCE node pool creation
        :rtype: :class:`~otcextensions.sdk.cce.v3.node_pool.NodePool`
        """
        count = kwargs.get('count')
        cce_cluster = kwargs.get('cluster')
        ecs_group = kwargs.get('ecs_group')
        flavor = kwargs.get('flavor')
        k8s_tags = kwargs.get('k8s_tags')
        lvm_config = kwargs.get('lvm_override_config')
        min_node_count = kwargs.get('min_node_count')
        max_node_count = kwargs.get('max_node_count')
        max_pods = kwargs.get('max_pods')
        name = kwargs.get('name')
        node_image_id = kwargs.get('node_image_id')
        os = kwargs.get('os')
        preinstall_script = kwargs.get('preinstall_script')
        postinstall_script = kwargs.get('postinstall_script')
        priority = kwargs.get('priority')
        public_key = kwargs.get('public_key')
        scale_down_cooldown_time = kwargs.get('scale_down_cooldown_time')
        ssh_key = kwargs.get('ssh_key')
        network = kwargs.get('network')
        tags = kwargs.get('tags')
        taints = kwargs.get('taints')

        volume_types = ['SAS', 'SATA', 'SSD']
        effects = ['NoSchedule', 'PrefereNoSchedule', 'NoExecute']

        metadata = {
            'name': name,
        }

        # Node template specs
        node_template = {
            'dataVolumes': {},
            'extendParam': {},
            'k8sTags': {},
            'login': {},
            'rootVolume': {},
            'userTags': [],
            'taints': [],
            'nodeNicSpec': {
                'primaryNic': {}
            }
        }

        if not (cce_cluster
                and flavor
                and os
                and name
                and network
                and ssh_key):
            raise ValueError('One or more of the following required '
                             'arguments are missing: cce_cluster, '
                             'flavor, name, network, os, ssh_key')
        node_template['flavor'] = flavor
        node_template['az'] = availability_zone
        if count:
            if not isinstance(count, int):
                raise ValueError('count is not an integer value')
            if count < 0:
                raise ValueError('count is 0 or lower')
            node_template['count'] = count
        node_template['login']['sshKey'] = ssh_key
        node_template['os'] = os
        if billing_mode and (billing_mode != 0):
            raise ValueError('billing_mode must be 0.')
        node_template['billingMode'] = billing_mode

        # Root volume specs
        if root_volume_size:
            if not isinstance(root_volume_size, int):
                raise ValueError('root_volume_size is not an integer value')
            if root_volume_size < 40:
                raise ValueError('Root volume size %s is lower than 40 GB.'
                                 % root_volume_size)
            node_template['rootVolume']['size'] = root_volume_size
        if root_volume_type.upper() not in volume_types:
            raise ValueError('Root volume type %s is not supported, use: %s'
                             % root_volume_type, volume_types)
        node_template['rootVolume']['volumetype'] = root_volume_type.upper()

        # Data volume specs
        volumes = []
        for item in data_volumes:
            if not (item['volumetype'] and item['size']):
                raise ValueError('One or both data volume keys: volumetype '
                                 'or size are missing.')
            if item['volumetype'] not in volume_types:
                raise ValueError('The data volumes volumetype %s must be '
                                 'one of the following choices: %s'
                                 % item['volumetype'], volume_types)
            if not isinstance(item['size'], int):
                try:
                    item['size'] = int(item['size'])
                except ValueError:
                    print('data_volume size %s cannot be converted into '
                          'integer value.' % item['size'])
            if not (100 <= item['size'] <= 32768):
                raise ValueError('The data volume size must be specified '
                                 'between 100 and 32768 GB.')
            if ('encrypted' in item) and item['encrypted']:
                if not item['cmk_id']:
                    raise ValueError('Parameter cmk_id is missing to use '
                                     'data volume encryption.')
                volumes.append({
                    'volumetype': item['volumetype'].upper(),
                    'size': item['size'],
                    'metadata': {
                        '__system__encrypted': '1',
                        '__system__cmkid': item['cmk_id']
                    }
                })
            else:
                volumes.append({
                    'volumetype': item['volumetype'].upper(),
                    'size': item['size']
                })
        if volumes:
            node_template['dataVolumes'] = volumes

        # extended parameter specs
        if lvm_config:
            lvm = lvm_config
            node_template['extendParam']['DockerLVMConfigOverride'] = lvm
        if max_pods:
            if not isinstance(max_pods, int):
                raise ValueError('max_pods is not an integer value')
            node_template['extendParam']['maxPods'] = max_pods
        if node_image_id:
            image = node_image_id
            node_template['extendParam']['alpha.cce/NodeImageID'] = image
        if postinstall_script:
            post = postinstall_script
            node_template['extendParam']['alpha.cce/postInstall'] = post
        if preinstall_script:
            pre = preinstall_script
            node_template['extendParam']['alpha.cce/preInstall'] = pre
        if public_key:
            node_template['extendParam']['publicKey'] = public_key
        if not node_template['extendParam']:
            del node_template['extendParam']

        # Tags
        if k8s_tags:
            if not isinstance(k8s_tags, dict):
                raise ValueError('k8s_tags is not a dict.')
            node_template['k8sTags'] = k8s_tags
        if tags:
            if not isinstance(tags, list):
                raise ValueError('tags parameter is not formatted as list.')
            if len(tags) > 20:
                ValueError('Parameter tags exceeds 20 list items.')
            for item in tags:
                # check for dict validity
                if not isinstance(item, dict):
                    raise ValueError('One or more tags items %s are not '
                                     'formatted as dicts.'
                                     % item)
                if not (set(['key', 'value']) <= item.keys()):
                    raise ValueError('The current tags list item %s has '
                                     'wrong format. Each dict item has to '
                                     'provide the keys: key and value.'
                                     % item)
            node_template['userTags'] = tags

        # Taints
        if taints:
            if not isinstance(taints, list):
                raise ValueError('taints parameter is not formatted as list')
            for item in taints:
                # Test taints list for conformity
                if not isinstance(item, dict):
                    raise ValueError('Each list item of taints parameter has '
                                     'to be a dict.')
                if not (set(['key', 'value', 'effect']) <= item.keys()):
                    raise ValueError('Each taints list item must provide '
                                     'the following keys: key, value, '
                                     'effect.')
                if not (1 <= len(item['key']) <= 63):
                    raise ValueError('taints key %s exceeds character '
                                     'range.'
                                     % item['key'])
                if not (1 <= len(item['value']) <= 63):
                    raise ValueError('taints value %s exceeds character '
                                     'range.'
                                     % item['value'])
                if item['effect'] not in effects:
                    raise ValueError('taints effect %s is different from the '
                                     'possible options: NoSchedule, '
                                     'PrefereNoSchedule, NoExecute.'
                                     % item['effect'])
            node_template['taints'] = taints

        # NIC specifications
        nw = self.network.find_network(
            network, ignore_missing=False)
        node_template['nodeNicSpec']['primaryNic']['subnet_id'] = nw.id

        # Node pool specs
        spec = {
            'autoscaling': {},
            'nodeManagement': {},
            'nodeTemplate': node_template
        }

        # Autoscaling specs
        if autoscaling_enabled:
            spec['autoscaling']['enable'] = autoscaling_enabled
            if min_node_count:
                if not isinstance(min_node_count, int):
                    raise ValueError('min_node_count is not an integer '
                                     'value.')
                spec['autoscaling']['minNodeCount'] = min_node_count
            if max_node_count:
                if not isinstance(max_node_count, int):
                    raise ValueError('max_node_count is not an integer '
                                     'value.')
                spec['autoscaling']['maxNodeCount'] = max_node_count
            if priority:
                if not isinstance(priority, int):
                    raise ValueError('priority is not an integer '
                                     'value.')
                if (priority > 1):
                    spec['autoscaling']['priority'] = priority
                else:
                    spec['autoscaling']['priority'] = 1
                if scale_down_cooldown_time:
                    if not isinstance(scale_down_cooldown_time, int):
                        raise ValueError('scale_down_cooldown_time is not an '
                                         'integer value.')
                    sdct = scale_down_cooldown_time
                    spec['autoscaling']['scaleDownCooldownTime'] = sdct
        else:
            del spec['autoscaling']

        # Node management specs
        if ecs_group:
            spec['nodeManagement']['serverGroupReference'] = ecs_group
        else:
            del spec['nodeManagement']

        # Node pool specs
        if initial_node_count:
            if not isinstance(initial_node_count, int):
                raise ValueError('initial_node_count is not an integer '
                                 'value.')
            spec['initialNodeCount'] = initial_node_count

        cluster = self.cce.find_cluster(
            name_or_id=cce_cluster,
            ignore_missing=True)
        if not cluster:
            raise ReferenceError('Cluster %s not found.' % cce_cluster)
        obj = self.cce.create_node_pool(
            cluster=cluster.id,
            metadata=metadata,
            spec=spec
        )

        return obj

    def delete_cce_node_pool(
        self,
        **kwargs
    ):
        """Delete CCE node pool

        :param str cluster: Name or ID of the CCE cluster
        :param str node_pool: Name or ID of the CCE node_pool
        """
        cluster = kwargs.get('cluster')
        node_pool = kwargs.get('node_pool')

        cluster = self.cce.find_cluster(
            name_or_id=cluster,
            ignore_missing=False)
        node_pool = self.cce.find_node_pool(
            cluster=cluster,
            node_pool=node_pool)

        self.cce.delete_node_pool(
            cluster=cluster,
            node_pool=node_pool
        )

        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''MRS clusters v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command
from datetime import datetime
from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)

CLUSTER_STATES = ['starting', 'running', 'terminated',
                  'failed', 'abnormal', 'terminating',
                  'frozen', 'scaling-out', 'scaling-in']
TIMESTAMP = '%Y-%m-%dT%H:%M:%S'

_formatters = {}


def _utc_to_timestamp(obj):
    obj = float(obj)
    return datetime.utcfromtimestamp(obj).strftime(TIMESTAMP)


def _flatten_cluster(obj):
    """Flatten the structure of the cluster into a single dict
    """
    data = {
        'id': obj.id,
        'name': obj.name,
        'status': obj.status,
        'region': obj.region,
        'cluster_type': obj.cluster_type,
        'availability_zone': obj.az_id,
        'version': obj.cluster_version,
        'tags': obj.tags,
        'created_at': _utc_to_timestamp(obj.created_at),
        'updated_at': _utc_to_timestamp(obj.updated_at),
        'billing_type': obj.billing_type,
        'vpc': obj.vpc,
        'core_node_flavor': obj.core_node_size,
        'external_ip': obj.external_ip,
        'internal_ip': obj.internal_ip,
        'master_num': obj.master_num,
        'core_num': obj.core_num,
        'master_node_size': obj.master_node_size,
        'core_node_size': obj.core_node_size,
        'component_list': obj.component_list,
        'deployment_id': obj.deployment_id,
        'instance_id': obj.instance_id,
        'vnc': obj.vnc,
        'project_id': obj.project_id,
        'volume_size': obj.volume_size,
        'volume_type': obj.volume_type,
        'subnet_id': obj.subnet_id,
        'subnet_name': obj.subnet_name,
        'security_group_id': obj.security_group_id,
        'non_master_security_group_id': obj.non_master_security_group_id,
        'safe_mode': obj.safe_mode,
        'key': obj.key,
        'master_ip': obj.master_ip,
        'preffered_private_ip': obj.preffered_private_ip,
        'charging_start_time': obj.charging_start_time,
        'task_node_groups': obj.task_node_groups,
        'node_groups': obj.node_groups,
        'bootstrap_scripts': obj.bootstrap_scripts,
        'scale': obj.scale
    }

    return data


def _get_columns(item):
    column_map = {}
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


def _normalize_tags(tags):
    result = ''
    for tag in tags:
        tag_parts = tag.split('=')
        if len(tag_parts) == 2 and tag_parts[1]:
            result += f'{tag_parts[0]}*{tag_parts[1]},'
        else:
            result += f'{tag_parts[0]},'
    return result[:-1]


def _add_tags_to_cluster_obj(obj, data, columns):
    res = ''
    tags = obj.tags[0].split(',')
    for tag in tags:
        tag_parts = tag.split('=')
        res += f'key={tag_parts[0]}, value={tag_parts[1]}\n'
    data += (res,)
    columns = columns + ('tags',)
    return data, columns


class ListCluster(command.Lister):
    _description = _('List MRS clusters')
    columns = (
        'id', 'name', 'status', 'region',
        'cluster_type', 'availability_zone', 'version'
    )

    def get_parser(self, prog_name):
        parser = super(ListCluster, self).get_parser(prog_name)
        parser.add_argument(
            '--project-id',
            metavar='<project_id>',
            help=_("Specifies the project ID."),
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            type=lambda s: s.lower(),
            choices=CLUSTER_STATES,
            help=_('Cluster status filter.')
        )
        parser.add_argument(
            '--tag',
            metavar='<tag>',
            action='append',
            help=_('Tag to assign to the server in KEY=VALUE format. '
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of records to return.')
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('Current page number.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.mrs

        query = {}

        if parsed_args.project_id:
            query['project_id'] = parsed_args.project_id
        if parsed_args.status:
            query['status'] = parsed_args.status
        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.marker:
            query['marker'] = parsed_args.marker
        if parsed_args.tag:
            query['tags'] = _normalize_tags(parsed_args.tag)

        data = client.clusters(**query)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_cluster(s), self.columns,
                 ) for s in data))
        return table


class ShowCluster(command.ShowOne):
    _description = _('Show single Cluster details')
    columns = (
        'ID',
        'name',
        'status',
        'region',
        'cluster_type',
        'availability_zone',
        'version',
        'created_at',
        'updated_at',
        'billing_type',
        'vpc',
        'core_node_flavor',
        'external_ip',
        'internal_ip',
        'master_num',
        'core_num',
        'master_node_size',
        'core_node_size',
        'deployment_id',
        'component_list',
        'instance_id',
        'vnc',
        'project_id',
        'volume_size',
        'volume_type',
        'subnet_id',
        'subnet_name',
        'security_group_id',
        'non_master_security_group_id',
        'safe_mode',
        'key',
        'master_ip',
        'preffered_private_ip',
        'charging_start_time',
        'task_node_groups',
        'node_groups',
        'bootstrap_scripts',
        'scale'
    )

    def get_parser(self, prog_name):
        parser = super(ShowCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the MRS cluster.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.mrs

        obj = client.find_cluster(
            name_or_id=parsed_args.cluster,
            ignore_missing=False
        )

        data = utils.get_dict_properties(
            _flatten_cluster(obj), self.columns)

        if obj.tags:
            data, self.columns = _add_tags_to_cluster_obj(
                obj, data, self.columns)
        return self.columns, data


class ListClusterHost(command.Lister):
    _description = _('Show the MRS Cluster Hosts')
    columns = (
        'id', 'name', 'status', 'flavor', 'type',
        'ip', 'mem', 'cpu', 'data_volume_size'
    )

    def get_parser(self, prog_name):
        parser = super(ListClusterHost, self).get_parser(prog_name)

        parser.add_argument(
            'cluster_id',
            metavar='<cluster_id>',
            help=_('id of the cluster.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.mrs

        query = {}

        if parsed_args.cluster_id:
            query['cluster_id'] = parsed_args.cluster_id

        data = client.hosts(**query)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table


class DeleteCluster(command.Command):
    _description = _('Delete Cluster')

    def get_parser(self, prog_name):
        parser = super(DeleteCluster, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            nargs='+',
            help=_('ID or Name of the cluster.')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.cluster:
            client = self.app.client_manager.mrs
            for cluster in parsed_args.cluster:
                client.delete_cluster(cluster=cluster, ignore_missing=False)


class UpdateCluster(command.ShowOne):
    _description = _('Update MRS Cluster')
    columns = (
        'ID',
        'name',
        'status',
        'region',
        'cluster_type',
        'availability_zone',
        'version',
        'created_at',
        'updated_at',
        'billing_type',
        'vpc',
        'core_node_flavor',
        'external_ip',
        'internal_ip',
        'master_num',
        'core_num',
        'master_node_size',
        'core_node_size',
        'deployment_id',
        'component_list',
        'instance_id',
        'vnc',
        'project_id',
        'volume_size',
        'volume_type',
        'subnet_id',
        'subnet_name',
        'security_group_id',
        'non_master_security_group_id',
        'safe_mode',
        'key',
        'master_ip',
        'preffered_private_ip',
        'charging_start_time',
        'task_node_groups',
        'node_groups',
        'bootstrap_scripts',
        'scale'
    )

    def get_parser(self, prog_name):
        parser = super(UpdateCluster, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the cluster.')
        )
        parser.add_argument(
            '--scale_type',
            metavar='<scale_type>',
            required=True,
            choices=['scale_in', 'scale_out'],
            help=_('Cluster scale-in or scale-out type.')
        )
        parser.add_argument(
            '--node_id',
            metavar='<node_id>',
            default='node_orderadd',
            help=_('ID of the newly added or removed node.')
        )
        parser.add_argument(
            '--instances',
            metavar='<instances>',
            type=int,
            required=True,
            help=_('Number of nodes to be added or removed.')
        )
        parser.add_argument(
            '--node_group',
            metavar='<node_group>',
            help=_('Node group to be scaled out or in.')
        )
        parser.add_argument(
            '--skip_bootstrap_scripts',
            metavar='<skip_bootstrap_scripts>',
            type=bool,
            help=_('indicates whether the bootstrap action specified '
                   'during cluster creation is performed on nodes '
                   'added during scale-out.')
        )
        parser.add_argument(
            '--scale_without_start',
            metavar='<scale_without_start>',
            type=bool,
            help=_('Whether to start components on the added '
                   'nodes after cluster scale-out.')
        )
        parser.add_argument(
            '--server_ids',
            metavar='<server_ids>',
            action='append',
            help=_('Task node to be deleted during task node scale-in.'
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--node_size',
            metavar='<node_size>',
            help=_('Instance specifications of a Task node.'
                   'For example: c2.2xlarge.linux.mrs')
        )
        parser.add_argument(
            '--data_volume_type',
            metavar='<data_volume_type>',
            choices=['SATA', 'SAS', 'SSD'],
            help=_('Data disk storage type of the Task node.')
        )
        parser.add_argument(
            '--data_volume_count',
            metavar='<data_volume_count>',
            type=int,
            help=_('Number of data disks of a Task node.'
                   'Value range: 1 to 10.')
        )
        parser.add_argument(
            '--data_volume_size',
            metavar='<data_volume_size>',
            type=int,
            help=_('Data disk storage space of a Task node.'
                   'Value range: 100 GB to 32,000 GB.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.mrs

        args_list = [
            'node_group',
            'skip_bootstrap_scripts',
            'scale_without_start',
            'server_ids',
        ]
        task_args_list = [
            'node_size',
            'data_volume_type',
            'data_volume_count',
            'data_volume_size',
        ]

        attrs = {
            'parameters': {
                'scale_type': parsed_args.scale_type,
                'node_id': parsed_args.node_id,
                'instances': parsed_args.instances
            },
        }
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs['parameters'][arg] = val

        node_info = {
            'task_node_info': {}
        }
        for arg in task_args_list:
            val = getattr(parsed_args, arg)
            if val:
                node_info['task_node_info'][arg] = val
        if node_info['task_node_info']:
            attrs['parameters'].update(node_info)

        cluster = client.find_cluster(
            name_or_id=parsed_args.cluster,
            ignore_missing=False
        )

        obj = client.update_cluster(cluster=cluster.id, **attrs)
        data = utils.get_dict_properties(
            _flatten_cluster(obj), self.columns)

        return self.columns, data



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.osclient.dws.v1 import cluster
from otcextensions.osclient.dws.v1 import snapshot
from otcextensions.tests.unit.osclient.dws.v1 import fakes


class TestListSnapshots(fakes.TestDws):
    objects = fakes.FakeSnapshot.create_multiple(3)

    column_list_headers = (
        'ID',
        'Name',
        'Type',
        'Cluster Id',
    )

    columns = (
        'id',
        'name',
        'type',
        'cluster_id',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.name,
                s.type,
                s.cluster_id,
            )
        )

    def setUp(self):
        super(TestListSnapshots, self).setUp()

        self.cmd = snapshot.ListSnapshots(self.app, None)

        self.client.snapshots = mock.Mock()
        self.client.api_mock = self.client.snapshots

    def test_list(self):
        arglist = []
        verifylist = []
        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateSnapshot(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()
    _snapshot = fakes.FakeSnapshot.create_one()

    columns = (
        'cluster_id',
        'created_at',
        'description',
        'id',
        'name',
        'size',
        'status',
        'type',
        'updated_at',
    )

    data = fakes.gen_data(_snapshot, columns)

    default_timeout = 900

    def setUp(self):
        super(TestCreateSnapshot, self).setUp()

        self.cmd = snapshot.CreateSnapshot(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.create_snapshot = mock.Mock(return_value=self._snapshot)
        self.client.get_snapshot = mock.Mock(return_value=self._snapshot)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_create(self):
        arglist = [
            self._cluster.name,
            'test-snapshot',
            '--description', 'test description',
            '--wait',
        ]
        verifylist = [
            ('cluster', self._cluster.name),
            ('name', 'test-snapshot'),
            ('description', 'test description'),
            ('wait', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'test-snapshot',
            'cluster_id': self._cluster.id,
            'description': 'test description',
        }
        self.client.create_snapshot.assert_called_with(**attrs)
        self.client.wait_for_cluster.assert_called_with(
            self._cluster.id, wait=self.default_timeout
        )
        self.assertEqual(self.columns, columns)


class TestShowSnapshot(fakes.TestDws):
    _snapshot = fakes.FakeSnapshot.create_one()

    columns = (
        'cluster_id',
        'created_at',
        'description',
        'id',
        'name',
        'size',
        'status',
        'type',
        'updated_at',
    )

    data = fakes.gen_data(_snapshot, columns)

    def setUp(self):
        super(TestShowSnapshot, self).setUp()

        self.cmd = snapshot.ShowSnapshot(self.app, None)

        self.client.find_snapshot = mock.Mock(return_value=self._snapshot)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._snapshot.id,
        ]

        verifylist = [
            ('snapshot', self._snapshot.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_snapshot.assert_called_with(
            self._snapshot.id, ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_dws_snapshot',
        ]

        verifylist = [
            ('snapshot', 'unexist_dws_snapshot'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_snapshot = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_snapshot.assert_called_with(
            'unexist_dws_snapshot', ignore_missing=False
        )


class TestRestoreSnapshot(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()
    _snapshot = fakes.FakeSnapshot.create_one()

    columns = (
        'action_progress',
        'availability_zone',
        'created_at',
        'endpoints',
        'enterprise_project_id',
        'flavor',
        'flavor_id',
        'floating_ip',
        'guest_agent_version',
        'id',
        'is_logical_cluster_enabled',
        'is_logical_cluster_initialed',
        'is_logical_cluster_mode',
        'maintenance_window',
        'name',
        'network_id',
        'nodes',
        'num_free_nodes',
        'num_nodes',
        'num_recent_events',
        'parameter_group',
        'port',
        'private_domain',
        'private_ip',
        'public_domain',
        'public_endpoints',
        'router_id',
        'security_group_id',
        'spec_version',
        'status',
        'sub_status',
        'tags',
        'task_status',
        'updated_at',
        'user_name',
        'version',
    )

    data = fakes.gen_data(_cluster, columns, cluster._formatters)

    default_timeout = 1800

    def setUp(self):
        super(TestRestoreSnapshot, self).setUp()

        self.cmd = snapshot.RestoreSnapshot(self.app, None)

        self.client.restore_snapshot = mock.Mock(return_value=self._cluster)
        self.client.wait_for_cluster = mock.Mock(return_value=True)
        self.client.get_cluster = mock.Mock(return_value=self._cluster)

    def test_restore(self):
        arglist = [
            'restored-cluster',
            '--snapshot-id', self._snapshot.id,
            '--router-id', 'router-uuid',
            '--network-id', 'network-uuid',
            '--security-group-id', 'sg-uuid',
            '--port', '9000',
            '--availability-zone', 'test-az',
            '--enterprise-project-id', 'eps-uuid',
            '--floating-ip', 'auto',
            '--wait',
        ]
        verifylist = [
            ('name', 'restored-cluster'),
            ('snapshot_id', self._snapshot.id),
            ('vpc_id', 'router-uuid'),
            ('subnet_id', 'network-uuid'),
            ('security_group_id', 'sg-uuid'),
            ('port', 9000),
            ('availability_zone', 'test-az'),
            ('enterprise_project_id', 'eps-uuid'),
            ('floating_ip', 'auto'),
            ('wait', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'restored-cluster',
            'vpc_id': 'router-uuid',
            'subnet_id': 'network-uuid',
            'security_group_id': 'sg-uuid',
            'port': 9000,
            'availability_zone': 'test-az',
            'enterprise_project_id': 'eps-uuid',
            'public_ip': {
                'public_bind_type': 'auto_assign',
                'eip_id': ''
            }
        }
        self.client.restore_snapshot.assert_called_with(
            self._snapshot.id, **attrs
        )
        self.client.wait_for_cluster.assert_called_with(
            self._cluster.id, self.default_timeout
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteSnapshot(fakes.TestDws):
    _snapshot = fakes.FakeSnapshot.create_multiple(2)

    def setUp(self):
        super(TestDeleteSnapshot, self).setUp()

        self.client.find_snapshot = mock.Mock(return_value=self._snapshot[0])
        self.client.delete_snapshot = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = snapshot.DeleteSnapshot(self.app, None)

    def test_delete(self):
        arglist = [
            self._snapshot[0].id,
        ]

        verifylist = [
            ('snapshot', [self._snapshot[0].id]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_snapshot.assert_called_with(
            self._snapshot[0].id, ignore_missing=False
        )
        self.client.delete_snapshot.assert_called_with(self._snapshot[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []
        snapshot_list = []

        for dws_snapshot in self._snapshot:
            snapshot_list.append(dws_snapshot.name)
            arglist.append(dws_snapshot.name)

        verifylist = [
            ('snapshot', snapshot_list),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for dws_snapshot in self._snapshot:
            calls.append(call(dws_snapshot.name, ignore_missing=False))
        self.client.find_snapshot.assert_has_calls(calls)
        self.client.delete_snapshot.assert_any_call(self._snapshot[0].id)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._snapshot[0].id,
            'unexist_dws_snapshot',
        ]
        verifylist = [
            ('snapshot', [self._snapshot[0].id, 'unexist_dws_snapshot']),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_results = [None, exceptions.CommandError]
        self.client.delete_snapshot = mock.Mock(
            side_effect=delete_mock_results
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 Snapshot(s) failed to delete.', str(e))

        self.client.delete_snapshot.assert_any_call(self._snapshot[0].id)



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Compute v2 Server action implementations"""

import getpass
import logging

from novaclient import api_versions
from osc_lib.cli import parseractions
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _

from otcextensions import sdk


LOG = logging.getLogger(__name__)


class SetServer(command.Command):
    _description = _("Set server properties")

    def get_parser(self, prog_name):
        parser = super(SetServer, self).get_parser(prog_name)
        parser.add_argument(
            'server',
            metavar='<server>',
            help=_('Server (name or ID)'),
        )
        parser.add_argument(
            '--name',
            metavar='<new-name>',
            help=_('New server name'),
        )
        parser.add_argument(
            '--root-password',
            action="store_true",
            help=_('Set new root password (interactive only)'),
        )
        parser.add_argument(
            "--property",
            metavar="<key=value>",
            action=parseractions.KeyValueAction,
            help=_('Property to add/change for this server '
                   '(repeat option to set multiple properties)'),
        )
        parser.add_argument(
            '--state',
            metavar='<state>',
            choices=['active', 'error'],
            help=_('New server state (valid value: active, error)'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('New server description (supported by '
                   '--os-compute-api-version 2.19 or above)'),
        )
        parser.add_argument(
            '--tag',
            metavar='<tag>',
            action='append',
            help=_('New Tag (supported by --os-compute-api-version 2.26 and '
                   'above')
        )
        return parser

    def take_action(self, parsed_args):

        compute_client = self.app.client_manager.compute
        server = utils.find_resource(
            compute_client.servers,
            parsed_args.server,
        )

        if parsed_args.name:
            server.update(name=parsed_args.name)

        if parsed_args.property:
            compute_client.servers.set_meta(
                server,
                parsed_args.property,
            )

        if parsed_args.state:
            server.reset_state(state=parsed_args.state)

        if parsed_args.root_password:
            p1 = getpass.getpass(_('New password: '))
            p2 = getpass.getpass(_('Retype new password: '))
            if p1 == p2:
                server.change_password(p1)
            else:
                msg = _("Passwords do not match, password unchanged")
                raise exceptions.CommandError(msg)

        if parsed_args.description:
            if server.api_version < api_versions.APIVersion("2.19"):
                msg = _("Description is not supported for "
                        "--os-compute-api-version less than 2.19")
                raise exceptions.CommandError(msg)
            server.update(description=parsed_args.description)

        if parsed_args.tag:
            if server.api_version < api_versions.APIVersion("2.26"):
                msg = _("Modifying tags is not supported for "
                        "--os-compute-api-version less than 2.26")
                raise exceptions.CommandError(msg)
            sdk.load(self.app.client_manager.sdk_connection)
            client = self.app.client_manager.sdk_connection.compute
            server = client.get_server(server.id)
            for tag in parsed_args.tag:
                server.add_tag(client, tag)


class UnsetServer(command.Command):
    _description = _("Unset server properties")

    def get_parser(self, prog_name):
        parser = super(UnsetServer, self).get_parser(prog_name)
        parser.add_argument(
            'server',
            metavar='<server>',
            help=_('Server (name or ID)'),
        )
        parser.add_argument(
            '--property',
            metavar='<key>',
            action='append',
            default=[],
            help=_('Property key to remove from server '
                   '(repeat option to remove multiple values)'),
        )
        parser.add_argument(
            '--description',
            dest='description',
            action='store_true',
            help=_('Unset server description (supported by '
                   '--os-compute-api-version 2.19 or above)'),
        )
        parser.add_argument(
            '--tag',
            metavar='<tag>',
            action='append',
            help=_('Unset tag (supported by --os-compute-api-version 2.26 and '
                   'above')
        )
        return parser

    def take_action(self, parsed_args):
        compute_client = self.app.client_manager.compute
        server = utils.find_resource(
            compute_client.servers,
            parsed_args.server,
        )

        if parsed_args.property:
            compute_client.servers.delete_meta(
                server,
                parsed_args.property,
            )

        if parsed_args.description:
            if compute_client.api_version < api_versions.APIVersion("2.19"):
                msg = _("Description is not supported for "
                        "--os-compute-api-version less than 2.19")
                raise exceptions.CommandError(msg)
            compute_client.servers.update(
                server,
                description="",
            )

        if parsed_args.tag:
            if compute_client.api_version < api_versions.APIVersion("2.26"):
                msg = _("Modifying tags is not supported for "
                        "--os-compute-api-version less than 2.26")
                raise exceptions.CommandError(msg)
            sdk.load(self.app.client_manager.sdk_connection)
            client = self.app.client_manager.sdk_connection.compute
            server = client.get_server(server.id)
            for tag in parsed_args.tag:
                server.remove_tag(client, tag)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v3 import cluster as _cluster

OS_HEADERS = {
    'Content-Type': 'application/json',
}

EXAMPLE_LIST = {
    'kind': 'Cluster',
    'apiVersion': 'v3',
    'items': [
        {
            'kind': 'Cluster',
            'apiVersion': 'v3',
            'metadata': {
                'name': 'mycluster-demo',
                'uid': '40c54866-38c5-11e9-b246-0255ac101413',
                'creationTimestamp': '2019-02-25 06:19:05.789462 +0000 UTC',
                'updateTimestamp': '2019-02-25 06:26:14.457426 +0000 UTC'
            },
            'spec': {
                'type': 'VirtualMachine',
                'flavor': 'cce.s1.small',
                'version': 'v1.13.10-r0',
                'az': 'eu-de-01',
                'supportIstio': True,
                'description': 'thisisademocluster',
                'hostNetwork': {
                    'vpc': 'a8cc62dc-acc2-47d0-9bfb-3b1d776c520b',
                    'subnet': '6d9e5355-85af-4a89-af28-243edb700db6'
                },
                'containerNetwork': {
                    'mode': 'overlay_l2',
                    'cidr': '172.16.0.0/16'
                },
                'authentication': {
                    'mode': 'rbac',
                    'authenticatingProxy': {}
                },
                'billingMode': 0,
                "kubeProxyMode": "iptables",
            },
            'status': {
                'phase': 'Available',
                'endpoints': {
                    'internal': 'https://192.168.0.129:5443',
                    'external_otc': 'https://40c54866-38c5-'
                }
            }
        }
    ]
}


class TestCluster(base.TestCase):

    def setUp(self):
        super(TestCluster, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

        self.sot = _cluster.Cluster()

        self.sot_expected = _cluster.Cluster(**EXAMPLE_LIST['items'][0])

    def test_basic(self):
        sot = _cluster.Cluster()
        self.assertEqual('', sot.resource_key)
        self.assertEqual('', sot.resources_key)
        self.assertEqual('/clusters',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST['items'][0]
        sot = _cluster.Cluster.existing(**obj)
        self.assertEqual(obj['metadata']['uid'], sot.id)
        self.assertEqual(obj['kind'], sot.kind)
        # self.assertEqual(obj['spec'], sot.spec)
        # self.assertEqual(obj['status'], sot.status)

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST)

        self.sess.get.return_value = mock_response

        result = list(
            self.sot.list(
                self.sess,
            )
        )

        self.sess.get.assert_called_once_with(
            '/clusters',
            params={},
            headers={
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        )

        expected_list = [
            self.sot_expected,
        ]

        self.assertEqual(
            set(i.id for i in expected_list),
            set(i.id for i in result))

    def test_get_status(self):
        data = EXAMPLE_LIST['items'][0]
        cluster = _cluster.Cluster(**data)

        self.assertEqual(data['status']['phase'], getattr(cluster,
                                                          'status.status'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestL7Rule(TestVlb):
    uuid_v4 = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestL7Rule, self).setUp()
        self.create_network()
        self.create_load_balancer()
        self.create_listener(protocol='HTTP')
        self.create_certificate()
        self.create_listener(
            protocol_port=443,
            protocol='HTTPS',
            name='sdk-vlb-test-r-lis-' + self.uuid_v4,
            additional=True,
            default_tls_container_ref=TestVlb.certificate.id)
        self.create_l7policy(TestVlb.additional_listener.id)
        self.create_l7rule()

    def test_01_list_l7Rules(self):
        l7p = list(self.client.l7_rules(TestVlb.l7policy))
        self.assertGreaterEqual(len(l7p), 0)

    def test_02_get_l7Policy(self):
        l7p = self.client.get_l7_rule(TestVlb.l7rule, TestVlb.l7policy)
        self.assertIsNotNone(l7p)

    def test_03_find_l7Rule(self):
        l7p = self.client.find_l7_rule(TestVlb.l7rule.id, TestVlb.l7policy)
        self.assertIsNotNone(l7p)

    def test_04_update_l7Rule(self):
        compare_type = 'STARTS_WITH'
        rule_value = '/testchange.com'
        l7p = self.client.update_l7_rule(
            TestVlb.l7rule,
            TestVlb.l7policy,
            compare_type=compare_type,
        )
        self.assertEqual(l7p['compare_type'], compare_type)
        l7p = self.client.update_l7_rule(
            TestVlb.l7rule,
            TestVlb.l7policy,
            value=rule_value,
        )
        self.assertEqual(l7p['value'], rule_value)
        # cleanup
        self.client.delete_l7_rule(TestVlb.l7rule, TestVlb.l7policy)
        self.client.delete_l7_policy(TestVlb.l7policy)
        self.client.delete_listener(TestVlb.additional_listener)
        self.client.delete_listener(TestVlb.listener)
        self.client.delete_certificate(TestVlb.certificate)
        self.client.delete_load_balancer(TestVlb.load_balancer)
        self.net_client.delete_ip(
            TestVlb.load_balancer.floating_ips[0]['publicip_id']
        )

        self.addCleanup(self.destroy_network, TestVlb.network)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.swr.v2.organization import Auth
from otcextensions.sdk.swr.v2 import _base


class Repository(resource.Resource):
    base_path = '/manage/namespaces/%(namespace)s/repos'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_fetch = True
    allow_list = True
    allow_commit = True

    commit_method = "PATCH"

    _query_mapping = resource.QueryParameters(
        'center', 'namespace', 'name', 'category', 'offset',
        'limit', 'order_column', 'order_type'
    )

    #: Organization namespace
    #: *Type:str*
    namespace = resource.URI('namespace')
    #: Image repository name
    #: *Type:str*
    repository = resource.Body('repository', type=str)
    #: Repository type
    #: The value can be app_server, linux, framework_app,
    #: database, lang, other, windows or arm.
    #: *Type:str*
    category = resource.Body('category', type=str)
    #: Brief description of the image repository
    #: *Type:str*
    description = resource.Body('description', type=str)
    #: Whether the repository is a public repository
    #: When the value is true, it indicates the repository is public.
    #: When the value is false, it indicates the repository is private.
    #: *Type:bool*
    is_public = resource.Body('is_public', type=bool)
    #: Image repository ID
    #: *Type:int*
    id = resource.Body('id')
    #: Organization ID
    #: *Type:int*
    ns_id = resource.Body('ns_id')
    #: Image repository name
    #: *Type:str*
    name = resource.Body('name')
    #: Image repository creator ID
    #: *Type:str*
    creator_id = resource.Body('creator_id')
    #: Image repository creator
    #: *Type:str*
    creator_name = resource.Body('creator_name')
    #: Image repository size
    #: *Type:str*
    size = resource.Body('size')
    #: Number of images in an image repository
    #: *Type:int*
    num_images = resource.Body('num_images')
    #: Download times of an image repository
    #: *Type:int*
    num_download = resource.Body('num_download')
    #: URL of the image repository logo image
    #: This field has been discarded and is left empty by default
    #: *Type:str*
    url = resource.Body('url')
    #: External image pull address.
    #: The format is {Repository address}/{Namespace name}/{Repository name}.
    #: *Type:str*
    path = resource.Body('path')
    #: Internal image pull address.
    #: The format is {Repository address}/{Namespace name}/{Repository name}.
    #: *Type:str*
    internal_path = resource.Body('internal_path')
    #: Time when an image repository is created. It is the UTC standard time
    #: *Type:str*
    created = resource.Body('created')
    #: Time when an image repository is updated. It is the UTC standard time
    #: *Type:str*
    updated = resource.Body('updated')
    #: Account ID
    #: *Type:str*
    domain_id = resource.Body('domain_id')
    #: Image sorting priority
    #: *Type:int*
    priority = resource.Body('priority')
    #: Image tag list
    #: *Type:list*
    tags = resource.Body('tags', type=list)
    #: Status
    #: *Type:bool*
    status = resource.Body('status')
    #: Total number of records
    #: *Type:int*
    total_range = resource.Body('total_range')


class Permission(_base.Resource):
    base_path = '/manage/namespaces/%(namespace)s/repos/%(repository)s/access'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    commit_method = "PATCH"

    requires_id = False

    #: Organization namespace
    #: *Type:str*
    namespace = resource.URI('namespace')
    #: Image repository name
    #: *Type:str*
    repository = resource.URI('repository')
    #: Information required for creating image permissions.
    #: *Type:list*
    permissions = resource.Body('permissions', type=list, list_type=Auth)
    #: Permission ID
    #: *Type:int*
    id = resource.Body('id')
    #: Image name
    #: *Type:str*
    name = resource.Body('name')
    #: Permissions of the current user
    #: *Type:dict*
    self_auth = resource.Body('self_auth', type=dict)
    #: Permissions of other users
    #: *Type:dict*
    others_auths = resource.Body('others_auths', type=list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class ServiceEvent(resource.Resource):
    base_path = "/services/%(service_id)s/events"

    resources_key = "events"

    allow_list = True

    _query_mapping = resource.QueryParameters(
        "event_type",
        "start_time",
        "end_time",
        "offset",
        "limit",
        "sort_by",
        "order",
    )

    # Properties
    #: Event information, including service operation records, key
    #:  actions during deployment, and deployment failure causes.
    event_info = resource.Body("event_info")
    #: Event type. Possible values are `normal` and `abnormal`, indicating
    #:  whether the event is normal or abnormal.
    event_type = resource.Body("event_type")
    #: Time when an event occurs. The value is milliseconds between the
    #:  current time and '1970.1.1 0:0:0 UTC'.
    occur_time = resource.Body("occur_time", type=int)
    #: Service ID.
    service_id = resource.URI("service_id")
    #: Service name.
    service_name = resource.Body("service_name")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ThrottlingExcludedPolicy(resource.Resource):
    base_path = (f'/apigw/instances/%(gateway_id)s/throttles/'
                 f'%(throttle_id)s/throttle-specials')

    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    resources_key = 'throttle_specials'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'object_type',
        'app_name', 'user'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Request throttling policy ID.
    throttle_id = resource.URI('throttle_id')

    # Maximum number of times an excluded object
    # can access an API within the throttling period.
    call_limits = resource.Body('call_limits', type=int)
    # Excluded app ID or account ID.
    object_id = resource.Body('object_id', type=str)
    # Excluded object type.
    # Enumeration values:
    # APP
    # USER
    object_type = resource.Body('object_type', type=str)

    # Attributes
    # Throttling period.
    apply_time = resource.Body('apply_time', type=str)
    # App name.
    app_name = resource.Body('app_name', type=str)
    # App ID.
    app_id = resource.Body('app_id', type=str)
    # Name of an app or a tenant to which the excluded configuration applies.
    object_name = resource.Body('object_name', type=str)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import binascii
import hashlib

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.kms.v1 import data_key as _key

EXAMPLE = {
    'key_id': '0d0466b0-e727-4d9c-b35d-f84bb474a37f',
    'plain_text': '8151014275E426C72EE7D44267EF11590DCE0089E19',
    'cipher_text': '020098009EEAFCE122CAA5927D2E020086F9548BA167',
}


class TestKey(base.TestCase):

    def setUp(self):
        super(TestKey, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()
        self.sot = _key.DataKey()

    def test_basic(self):
        sot = _key.DataKey()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/kms', sot.base_path)
        self.assertEqual('/kms/create-datakey', sot.create_path)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertTrue(sot.allow_update)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = _key.DataKey.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['key_id'], sot.key_id)
        self.assertEqual(EXAMPLE['plain_text'], sot.plain_text)
        self.assertEqual(EXAMPLE['cipher_text'], sot.cipher_text)

    def test_create(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = EXAMPLE.copy()

        self.sess.post.return_value = mock_response

        key = {
            'key_id': 'key',
            'encryption_context': {
                'a': 'b'
            },
            'datakey_length': 200,
            'sequence': 'seq',
        }

        sot = _key.DataKey(**key)
        result = sot.create(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        self.assertEqual('/kms/create-datakey', call_args[0][0])
        self.assertDictEqual(key, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['key_id'], sot.key_id)
        self.assertEqual(EXAMPLE['plain_text'], sot.plain_text)
        self.assertEqual(EXAMPLE['cipher_text'], sot.cipher_text)

    def test_create_wo_plain(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'key_id': EXAMPLE['key_id'],
            'cipher_text': EXAMPLE['cipher_text'],
        }

        self.sess.post.return_value = mock_response

        key = {
            'key_id': 'key',
            'encryption_context': {
                'a': 'b'
            },
            'datakey_length': 200,
            'sequence': 'seq',
        }

        sot = _key.DataKey(**key)
        result = sot.create_wo_plain(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        self.assertEqual(
            '/kms/create-datakey-without-plaintext',
            call_args[0][0])
        self.assertDictEqual(key, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['key_id'], sot.key_id)
        self.assertIsNone(sot.plain_text)
        self.assertEqual(EXAMPLE['cipher_text'], sot.cipher_text)

    def test_encrypt(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'key_id': EXAMPLE['key_id'],
            'cipher_text': EXAMPLE['cipher_text'],
            'datakey_length': 10
        }

        self.sess.post.return_value = mock_response

        plain_key = \
            '0000000000000000000000000000000000000000000000000000000000000000'
        hash = hashlib.sha256()
        hex_data = hex_data = binascii.unhexlify(plain_key)
        hash.update(bytearray(hex_data))
        digest = hash.hexdigest()
        encr_key = plain_key + digest

        key = {
            'key_id': EXAMPLE['key_id'],
            'encryption_context': {'a': 'b'},
            'plain_text': plain_key,
            'sequence': 'seq',
        }

        sot = _key.DataKey.existing(**key)
        result = sot.encrypt(self.sess)

        expected_json = {
            'key_id': EXAMPLE['key_id'],
            'encryption_context': {'a': 'b'},
            'plain_text': encr_key,
            'datakey_plain_length': len(hex_data),
            'sequence': 'seq'
        }

        call_args = self.sess.post.call_args_list[0]

        self.assertEqual('/kms/encrypt-datakey', call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['key_id'], sot.key_id)
        self.assertEqual(10, sot.datakey_length)
        self.assertEqual(EXAMPLE['cipher_text'], sot.cipher_text)

    def test_decrypt(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'data_key': EXAMPLE['plain_text'],
            'datakey_length': 10
        }

        self.sess.post.return_value = mock_response

        key = {
            'key_id': EXAMPLE['key_id'],
            'encryption_context': {'a': 'b'},
            'cipher_text': EXAMPLE['cipher_text'],
            'datakey_cipher_length': 200,
            'sequence': 'seq',
        }

        sot = _key.DataKey.existing(**key)
        result = sot.decrypt(self.sess)

        expected_json = key

        call_args = self.sess.post.call_args_list[0]

        self.assertEqual('/kms/decrypt-datakey', call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['key_id'], sot.key_id)
        self.assertEqual(EXAMPLE['plain_text'], sot.plain_text)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from osc_lib import exceptions

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import training_job
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes

_COLUMNS = (
    "created_at",
    "duration",
    "id",
    "job_description",
    "job_id",
    "job_name",
    "name",
    "status",
    "user",
    "version_count",
    "version_id",
)


class TestListTrainingJobs(fakes.TestModelartsv1):
    objects = fakes.FakeTrainingJob.create_multiple(3)

    column_list_headers = (
        "Job Id",
        "Job Name",
        "Status",
        "Version Id",
        "Version Count",
        "Created At",
        "Duration",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.job_id,
                s.job_name,
                training_job.JobStatus(s.status),
                s.version_id,
                s.version_count,
                cli_utils.UnixTimestampFormatter(s.created_at),
                s.duration,
            )
        )

    def setUp(self):
        super(TestListTrainingJobs, self).setUp()

        self.cmd = training_job.ListTrainingJobs(self.app, None)

        self.client.training_jobs = mock.Mock()
        self.client.api_mock = self.client.training_jobs

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--limit",
            "1",
            "--offset",
            "2",
            "--status",
            "10",
            "--sort-by",
            "job_name",
            "--order",
            "asc",
            "--search-content",
            "test-job-name",
            "--workspace-id",
            "0",
        ]

        verifylist = [
            ("limit", 1),
            ("offset", 2),
            ("status", 10),
            ("sort_by", "job_name"),
            ("order", "asc"),
            ("search_content", "test-job-name"),
            ("workspace_id", "0"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            offset=2,
            status=10,
            sort_by="job_name",
            order="asc",
            search_content="test-job-name",
            workspace_id="0",
        )


class TestCreateTrainingJob(fakes.TestModelartsv1):
    _data = fakes.FakeTrainingJob.create_one()
    columns = _COLUMNS

    data = fakes.gen_data(_data, columns, training_job._formatters)

    def setUp(self):
        super(TestCreateTrainingJob, self).setUp()

        self.cmd = training_job.CreateTrainingJob(self.app, None)

        self.client.create_training_job = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            "test-trainingjob",
            "--description",
            "1",
            "--workspace-id",
            "2",
            "--worker-server-num",
            "3",
            "--app-url",
            "4",
            "--boot-file-url",
            "5",
            "--log-url",
            "6",
            "--data-url",
            "7",
            "--dataset-id",
            "8",
            "--dataset-version",
            "9",
            "--spec-id",
            "11",
            "--engine-id",
            "12",
            "--model-id",
            "13",
            "--train-url",
            "14",
            "--parameter",
            "label=k1,value=v1",
            "--parameter",
            "label=k2,value=v2",
            "--nfs",
            "id=123,src_path=a,dest_path=b,read_only=false",
            "--host-path",
            "src_path=x,dest_path=y,read_only=false",
            "--no-create-version",
        ]
        verifylist = [
            ("name", "test-trainingjob"),
            ("job_desc", "1"),
            ("workspace_id", "2"),
            ("worker_server_num", 3),
            ("app_url", "4"),
            ("boot_file_url", "5"),
            ("log_url", "6"),
            ("data_url", "7"),
            ("dataset_id", "8"),
            ("dataset_version_id", "9"),
            ("spec_id", 11),
            ("engine_id", 12),
            ("model_id", 13),
            ("train_url", "14"),
            (
                "parameter",
                [
                    {"label": "k1", "value": "v1"},
                    {"label": "k2", "value": "v2"},
                ],
            ),
            (
                "nfs",
                [
                    {
                        "id": "123",
                        "src_path": "a",
                        "dest_path": "b",
                        "read_only": "false",
                    }
                ],
            ),
            (
                "host_path",
                [
                    {
                        "src_path": "x",
                        "dest_path": "y",
                        "read_only": "false",
                    }
                ],
            ),
            ("no_create_version", True),
        ]
        # Verify cm is triggereg with default parameters
        self.check_parser(self.cmd, arglist, verifylist)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            "job_name": "test-trainingjob",
            "config": {
                "worker_server_num": 3,
                "app_url": "4",
                "boot_file_url": "5",
                "parameter": [
                    {"label": "k1", "value": "v1"},
                    {"label": "k2", "value": "v2"},
                ],
                "data_url": "7",
                "dataset_id": "8",
                "dataset_version_id": "9",
                "spec_id": 11,
                "engine_id": 12,
                "model_id": 13,
                "train_url": "14",
                "log_url": "6",
                "create_version": False,
                "volumes": [
                    {
                        "nfs": {
                            "id": "123",
                            "src_path": "a",
                            "dest_path": "b",
                            "read_only": "false",
                        }
                    },
                    {
                        "host_path": {
                            "src_path": "x",
                            "dest_path": "y",
                            "read_only": "false",
                        }
                    },
                ],
            },
            "job_desc": "1",
            "workspace_id": "2",
        }
        self.client.create_training_job.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateTrainingJob(fakes.TestModelartsv1):
    _data = fakes.FakeTrainingJob.create_one()

    columns = _COLUMNS

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateTrainingJob, self).setUp()

        self.cmd = training_job.UpdateTrainingJob(self.app, None)

        self.client.update_training_job = mock.Mock(return_value=self._data)

    def test_update(self):
        arglist = [
            "1234",
            "--description",
            "New Description",
        ]
        verifylist = [
            ("jobId", 1234),
            ("description", "New Description"),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.update_training_job.assert_called_with(
            1234, "New Description"
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteTrainingJob(fakes.TestModelartsv1):
    def setUp(self):
        super(TestDeleteTrainingJob, self).setUp()

        self.client.delete_training_job = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = training_job.DeleteTrainingJob(self.app, None)

    def test_delete(self):
        arglist = ["123"]

        verifylist = [
            ("jobId", [123]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_training_job.assert_called_with(123)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = [
            "1234",
            "9876",
        ]

        verifylist = [
            ("jobId", [1234, 9876]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        delete_calls = [call(1234), call(9876)]
        self.client.delete_training_job.assert_has_calls(delete_calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            "1234",
            "5544",  # non-existing-job-id
        ]

        verifylist = [
            ("jobId", [1234, 5544]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_results = [None, exceptions.CommandError]
        self.client.delete_training_job = mock.Mock(
            side_effect=delete_mock_results
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                "1 of 2 training job(s) failed to delete.", str(e)
            )
